/*
 * H_Common_STM32F4_UART.h
 *
 *  Created on: 2024/11/18
 *      Author: egaono-electronics
 */

#ifndef H_COMMON_STM32F4_UART_H_
#define H_COMMON_STM32F4_UART_H_

#include "Dev_Conf.h"

#include def_Header_HAL



//------------------------------------------------------
// Handle UARTx用パラメータの構造体
//------------------------------------------------------
struct sParam_Handle_UARTx{
  uint32_t uint32_BaudRate_UART;
  uint32_t uint32_Parity_UART;

  uint32_t uint32_BaudRate;
  uint32_t uint32_Parity;

  uint8_t uint8_aBuffer_UART[16];

  uint8_t *puint8_aTx_Buffer;  // 送信Bufferのポインタ
  uint8_t *puint8_aRx_Buffer;  // 受信Bufferのポインタ

  // UART通信処理用 受信データBuffer
  uint8_t *puint8_aReceiveData_UART;

  volatile uint16_t uint16_vTx_Transfer;  // 送信Bufferサイズ
  volatile uint16_t uint16_vRx_Receive;   // 受信Bufferサイズ

  volatile uint16_t uint16_vTx_Counter;
  volatile uint16_t uint16_vRx_Counter;

  volatile uint16_t uint16_vRx_ReadPointer;
  volatile uint16_t uint16_vRx_WritePointer;
  volatile uint16_t uint16_vRx_WritePointerSave;

  volatile uint8_t uint8_vRx_Delimiter;

  volatile int16_t int16_vUART_ReceiveTimeOutVal; // 受信TimeOut値
  volatile int16_t int16_vUART_ReceiveTimeOut;    // 受信TimeOut Counter
  volatile int16_t int16_vUART_ReceiveEndTimeVal; // 受信終了Timer値
  volatile int16_t int16_vUART_ReceiveEndTime;    // 受信終了Timer Counter


  volatile int16_t int16_vUART_InitDone; // 初期化済みフラグ

  volatile int16_t int16_vUART_Error;
};



//----------------------------------------------------------
// Handle UARTx用パラメータの構造体を初期化
//----------------------------------------------------------
// 引数:
// uint8_t *puint8_aTx_Buffer : 送信Bufferのポインタ
// uint8_t *puint8_aRx_Buffer : 受信Bufferのポインタ

//  uint8_t *puint8_aReceiveData_UART :
//                      UART通信処理用 受信データBuffer

// volatile uint16_t uint16_vTx_Transfer : 送信Bufferサイズ
// volatile uint16_t uint16_vRx_Receive : 受信Bufferサイズ

// struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
//              Handle UART用パラメータの構造体のポインタ


// 戻り値:
//    なし
//----------------------------------------------------------
void Initialize_Parameter_UARTx(uint8_t *puint8_aTx_Buffer,
                        uint8_t *puint8_aRx_Buffer,
                        uint8_t *puint8_aReceiveData_UART,
                        volatile uint16_t uint16_vTx_Transfer,
                        volatile uint16_t uint16_vRx_Receive,
        struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx);



//----------------------------------------------------------
// UARTx 初期化
//----------------------------------------------------------
// 引数:
// USART_TypeDef* USARTx :
//    where x can be 1, 2, 3, 4, 5 or 6
//    to select the USART or UART peripheral.

// UART_HandleTypeDef *pHandle_UARTx :
//  pointer to a UART_HandleTypeDef structure that contains
//  the configuration information
//                      for the specified UART module.

// uint32_t uint32_BaudRate : BaudRate
// uint32_t uint32_Parity : Parity
// uint8_t *puint8_aBuffer_UARTx : 受信Bufferのポインタ

// volatile int16_t *pint16_vUARTx_InitDone :
//                          初期化済みフラグ変数のポインタ


// 戻り値:
//    0 : OK
//    1 : NG
//----------------------------------------------------------
int16_t Initialize_UARTx(USART_TypeDef* USARTx,
                          UART_HandleTypeDef *pHandle_UARTx,
                          uint32_t uint32_BaudRate,
                          uint32_t uint32_Parity,
                          uint8_t *puint8_aBuffer_UARTx,
                  volatile int16_t *pint16_vUARTx_InitDone);



//----------------------------------------------------------
// UARTx 送信処理 : 送信Bufferに送信データをセットして、
//          送信データ数をセットした状態でCallしてください。
//----------------------------------------------------------
// 送信Buffer : pstructParam_Handle_UARTx->uint8_aTx_Buffer
// 送信データ数 : pstructParam_Handle_UARTx->uint16_vTx_Transfer
//----------------------------------------------------------
// 引数:
// uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

// UART_HandleTypeDef *pHandle_UARTx :
//  pointer to a UART_HandleTypeDef structure that contains
//  the configuration information
//                      for the specified UART module.

// struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
//              Handle UART用パラメータの構造体のポインタ


// 戻り値:
//   -1 : 処理中
//    0 : 送信終了
//    1 : エラー
//----------------------------------------------------------
int16_t Send_UARTx(uint8_t *puint8_JobNum,
                   UART_HandleTypeDef *pHandle_UARTx,
        struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx);



//----------------------------------------------------------
// UARTx 受信処理
//----------------------------------------------------------
// 引数:
// uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

// uint8_t *puint8_aReceiveBuffer :
//                      受信データを格納するBufferのポインタ

// struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
//              Handle UART用パラメータの構造体のポインタ


// 戻り値:
//    -2 : エラー発生
//    -1 : 受信処理中
//     0 : 受信なし
// 1以上 : 受信byte数
//----------------------------------------------------------
int16_t Receive_UARTx(uint8_t *puint8_JobNum,
                      uint8_t *puint8_aReceiveBuffer,
        struct sParam_Handle_UARTx *pstructParam_Handle_UARTx);



//----------------------------------------------------------
// 受信データを抜き出す。
//----------------------------------------------------------
// 受信Bufferの読み込み開始位置と書き込み開始位置を使用して
// 受信データを抜き出した後、読み込み開始位置を更新する。
//----------------------------------------------------------
// 引数:
// struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
//              Handle UART用パラメータの構造体のポインタ

// uint8_t *puint8_aExtractData :
//              抜き出したデータを格納するBufferのポインタ

// 戻り値:
//     0 : データなし
// 1以上 : 受信データ数
//----------------------------------------------------------
uint16_t Extract_Receive_Data(
            struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx,
                              uint8_t *puint8_aExtractData);



#endif /* H_COMMON_STM32F4_UART_H_ */
