/*
 * Communication_Test_UARTx.c
 *
 *  Created on: 2025/01/29
 *      Author: egaono-electronics
 */

#include "Communication_Test_UARTx.h"

#include "Execute_Command_LED.h"


#include <string.h>



//----------------------------------------------------------
// UARTx通信パラメータ初期化
//----------------------------------------------------------
// 引数:
// USART_TypeDef* USARTx :
//    where x can be 1, 2, 3, 4, 5 or 6
//    to select the USART or UART peripheral.

// UART_HandleTypeDef *pHandle_UARTx :
//  pointer to a UART_HandleTypeDef structure that contains
//  the configuration information
//                      for the specified UART module.

// uint32_t uint32_BaudRate : 通信速度

// struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
//              Handle UART用パラメータの構造体のポインタ


// 戻り値:
//    なし
//----------------------------------------------------------
void Initialize_Communication_UARTx(USART_TypeDef* USARTx,
                        UART_HandleTypeDef *pHandle_UARTx,
                        uint32_t uint32_BaudRate,
        struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx)
{
  //------------------------------------------------------------
  // UART パラメータ初期化
  //------------------------------------------------------------

  //------------------------------------------------------------
  // 受信終了タイマのタイムアウトで受信終了する場合 :
  //                                    受信終了タイマ値設定
  //------------------------------------------------------------
  pstructParam_Handle_UARTx->int16_vUART_ReceiveEndTimeVal = 50;
                                    // 受信終了タイマ 50mSec :
        // 受信データが途切れてから受信終了タイマが0になったら終了


  // Baudrateセット
  pstructParam_Handle_UARTx->uint32_BaudRate_UART = uint32_BaudRate;


  //------------------------------------------------------------
  // UARTx初期化
  //------------------------------------------------------------
  // 引数:
  // USART_TypeDef* USARTx :
  //  where x can be 1, 2, 3, 4, 5 or 6 to select
  //  the USART or UART peripheral.

  // UART_HandleTypeDef *pHandle_UARTx :
  //   pointer to a UART_HandleTypeDef structure that contains
  //   the configuration information for the specified UART module.

  // uint32_t uint32_BaudRate : BaudRate
  // uint32_t uint32_Patity : Parity
  // uint8_t *puint8_aBuffer_UARTx : 受信Bufferのポインタ

  // volatile int16_t *pint16_vUARTx_InitDone :
  //                            初期化済みフラグ変数のポインタ


  // 戻り値:
  //    0 : OK
  //    1 : NG
  //------------------------------------------------------------
  Initialize_UARTx(USARTx,
                   pHandle_UARTx,
                   uint32_BaudRate,
                   (uint32_t)UART_PARITY_NONE,
                   pstructParam_Handle_UARTx->uint8_aBuffer_UART,
                   &pstructParam_Handle_UARTx->int16_vUART_InitDone);
}



//----------------------------------------------------------
// UARTx通信試験
//----------------------------------------------------------
// 処理内容:

//  1. UARTの初期化がされていない場合、初期化します。
//  2. 受信が発生するのを待ち、受信データを取得します。
//  3. 受信データを判定して、内容に従い処理を行います。
//  4. 受信データによる処理の結果、
//          送信データがセットされた場合、送信処理を行います。
//----------------------------------------------------------
// 引数:
// uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

// USART_TypeDef* USARTx :
//    where x can be 1, 2, 3, 4, 5 or 6
//    to select the USART or UART peripheral.

// UART_HandleTypeDef *pHandle_UARTx :
//  pointer to a UART_HandleTypeDef structure that contains
//  the configuration information
//                      for the specified UART module.

// uint8_t *puint8_aTx_Buffer : 送信Bufferのポインタ
// uint8_t *puint8_aRx_Buffer : 受信Bufferのポインタ

//  uint8_t *puint8_aReceiveData_UART :
//              UART通信処理用 受信データBufferのポインタ

// volatile uint16_t uint16_vTx_Transfer : 送信Bufferサイズ
// volatile uint16_t uint16_vRx_Receive : 受信Bufferサイズ

// struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
//              Handle UART用パラメータの構造体のポインタ


// 戻り値:
//   -1 : 処理中
//    0 : OK終了
//    1 : NG終了
//----------------------------------------------------------
int16_t Communication_Test_UARTx(uint8_t *puint8_JobNum,
                        USART_TypeDef* USARTx,
                        UART_HandleTypeDef *pHandle_UARTx,
                        uint8_t *puint8_aTx_Buffer,
                        uint8_t *puint8_aRx_Buffer,
                        uint8_t *puint8_aReceiveData_UART,
                        volatile uint16_t uint16_vTx_Transfer,
                        volatile uint16_t uint16_vRx_Receive,
        struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx)
{
  //----------------------------------------------------
  // 呼び出す関数の引数に使用する処理番号は、
  // 必ず static の変数宣言をしてください。
  //----------------------------------------------------
  static uint8_t STC_uint8_JobNum_UART;  // UART処理番号


  int16_t int16_ReceiveLength;

  int16_t int16_Return;


  switch(*puint8_JobNum)
  {
  case 0:
    //--------------------------------------------------
    // 1回だけ case 0: を実行します。

    // その後は、case 1: ～ case 3: をループします。
    //--------------------------------------------------


    //----------------------------------------------------------
    // Handle UARTx用パラメータの構造体を初期化
    //----------------------------------------------------------
    // 引数:
    // uint8_t *puint8_aTx_Buffer : 送信Bufferのポインタ
    // uint8_t *puint8_aRx_Buffer : 受信Bufferのポインタ

    //  uint8_t *puint8_aReceiveData_UART :
    //                      UART通信処理用 受信データBuffer

    // volatile uint16_t uint16_vTx_Transfer : 送信Bufferサイズ
    // volatile uint16_t uint16_vRx_Receive : 受信Bufferサイズ

    // struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
    //              Handle UART用パラメータの構造体のポインタ


    // 戻り値:
    //    なし
    //----------------------------------------------------------
    Initialize_Parameter_UARTx(puint8_aTx_Buffer,
                               puint8_aRx_Buffer,
                               puint8_aReceiveData_UART,
                               uint16_vTx_Transfer,
                               uint16_vRx_Receive,
                               pstructParam_Handle_UARTx);


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 1:
    if(pstructParam_Handle_UARTx->int16_vUART_InitDone == 0){
      //--------------------------------------------------------
      // UARTx通信パラメータ初期化
      //--------------------------------------------------------
      // 引数:
      // USART_TypeDef* USARTx :
      //    where x can be 1, 2, 3, 4, 5 or 6
      //    to select the USART or UART peripheral.

      // UART_HandleTypeDef *pHandle_UARTx :
      //  pointer to a UART_HandleTypeDef structure that contains
      //  the configuration information
      //                      for the specified UART module.

      // uint32_t uint32_BaudRate : 通信速度

      // struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
      //              Handle UART用パラメータの構造体のポインタ


      // 戻り値:
      //    0 : OK
      //    1 : NG
      //--------------------------------------------------------
      Initialize_Communication_UARTx(USARTx,
                                     pHandle_UARTx,
                                     (uint32_t)115200,
                                     pstructParam_Handle_UARTx);
    }


    STC_uint8_JobNum_UART = 0;  // UART処理番号初期化


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 2:
    //----------------------------------------------------------
    // UARTx 受信処理
    //----------------------------------------------------------
    // uint8_t *puint8_JobNum : 処理番号を格納する変数のポインタ

    // uint8_t *puint8_aReceiveBuffer :
    //                      受信データを格納するBufferのポインタ

    // struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
    //              Handle UART用パラメータの構造体のポインタ


    // 戻り値:
    //    -2 : エラー発生
    //    -1 : 受信なし
    //     0 : 受信なし
    // 1以上 : 受信byte数
    //----------------------------------------------------------
    int16_ReceiveLength = Receive_UARTx(&STC_uint8_JobNum_UART,
                                        puint8_aReceiveData_UART,
                                        pstructParam_Handle_UARTx);
    if(int16_ReceiveLength <= 0){
      if(int16_ReceiveLength ==  -2){
        //--------------------------
        // エラー発生
        //--------------------------
        *puint8_JobNum = 0;  // 処理番号初期化
        int16_Return = -2;   // エラー発生
        break;  // エラー終了
      }

      //-------------------
      // 受信なし
      //-------------------
      int16_Return = -1;  // 処理継続 : 現在の処理番号をループ
      break;  // 処理番号維持
    }


    //----------------------------------------------------------
    // 受信コマンドを実行する。 : LED点滅
    //----------------------------------------------------------
    // 引数:
    // uint16_t uint16_ReceiveLength : 受信データ数
    // uint8_t *puint8_aReceiveData :
    //              受信データが格納されているBufferのポインタ
    // uint8_t *puint8_aSendData :
    //              応答送信データを格納するBufferのポインタ

    // struct sParam_Blink_LED_ST *pstructParam_Blink_LED_ST :
    //              ST LED点滅用パラメータの構造体のポインタ


    // 戻り値:応答送信データ数
    //----------------------------------------------------------
    pstructParam_Handle_UARTx->uint16_vTx_Transfer
        = Execute_Command_LED((uint16_t)int16_ReceiveLength,
                              puint8_aReceiveData_UART,
                    pstructParam_Handle_UARTx->puint8_aTx_Buffer,
                              &GLB_structParam_Blink_LED_ST);


    (*puint8_JobNum)++;  // 次の処理番号に移行する。
    int16_Return = -1;   // 処理継続
    break;

  case 3:
    // 応答データ送信
    //----------------------------------------------------------
    // UARTx 送信処理
    //----------------------------------------------------------
    // 引数:
    // uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

    // UART_HandleTypeDef *pHandle_UARTx :
    //  pointer to a UART_HandleTypeDef structure that contains
    //  the configuration information
    //                      for the specified UART module.

    // struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
    //              Handle UART用パラメータの構造体のポインタ


    // 戻り値:
    //   -1 : 処理中
    //    0 : 送信終了
    //    1 : エラー
    //----------------------------------------------------------
    int16_Return = Send_UARTx(&STC_uint8_JobNum_UART,
                              pHandle_UARTx,
                              pstructParam_Handle_UARTx);
    if(int16_Return == -1){
      // 処理中
      break;  // 処理番号維持 : 処理継続 : 現在の処理番号をループ
    }

    *puint8_JobNum = 1;  // 処理番号 1 に移行する。: case 1:
                         // case 1: ～ case 3: をループ
    // int16_Return : 結果
    break;

  default:
    *puint8_JobNum = 0;  // 処理番号初期化
    int16_Return = 1;    // NG
    break;
  }

  return(int16_Return);
}



//----------------------------------------------------------
// UARTx通信試験 : 送信割り込み処理
//----------------------------------------------------------
// 引数:
// struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
//              Handle UART用パラメータの構造体のポインタ


// 戻り値:
//    なし
//----------------------------------------------------------
void Test_UARTx_TxCpltCallback(
        struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx)
{
    pstructParam_Handle_UARTx->uint16_vTx_Counter
        = pstructParam_Handle_UARTx->uint16_vTx_Transfer;
}



//----------------------------------------------------------
// UARTx通信試験 : 受信割り込み処理
//----------------------------------------------------------
// 引数:
// UART_HandleTypeDef *huart : UART_Handle構造体のポインタ

// struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
//              Handle UART用パラメータの構造体のポインタ


// 戻り値:
//    なし
//----------------------------------------------------------
void Test_UARTx_RxCpltCallback(UART_HandleTypeDef *huart,
        struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx)
{
    pstructParam_Handle_UARTx->int16_vUART_ReceiveTimeOut = -1;
                                                    // TimeOut OFF
    pstructParam_Handle_UARTx->int16_vUART_ReceiveEndTime
        = pstructParam_Handle_UARTx->int16_vUART_ReceiveEndTimeVal;
                                              // 受信終了Timerセット


    // 書き込み位置の値が異常値でないかチェック
    if(pstructParam_Handle_UARTx->uint16_vRx_WritePointer
                >= pstructParam_Handle_UARTx->uint16_vRx_Receive){
      // 書き込み位置の値が異常のとき 0 にする。
      pstructParam_Handle_UARTx->uint16_vRx_WritePointer = 0;
    }


    // 受信データ数チェック
    if(pstructParam_Handle_UARTx->uint16_vRx_Counter
                < pstructParam_Handle_UARTx->uint16_vRx_Receive){
      // 受信データ数がBufferサイズを超えていない場合、
      // 受信データをセットする。
      pstructParam_Handle_UARTx->puint8_aRx_Buffer[
            pstructParam_Handle_UARTx->uint16_vRx_WritePointer]
                = pstructParam_Handle_UARTx->uint8_aBuffer_UART[0];

      pstructParam_Handle_UARTx->uint16_vRx_WritePointer++;
                                    // 書き込み位置をインクリメント

      // 書き込み位置チェック
      if(pstructParam_Handle_UARTx->uint16_vRx_WritePointer
                >= pstructParam_Handle_UARTx->uint16_vRx_Receive){
        // 書き込み位置が Bufferサイズに達した場合、
        // 書き込み位置を 0 にする。 : Ring Buffer
        pstructParam_Handle_UARTx->uint16_vRx_WritePointer = 0;
      }


      pstructParam_Handle_UARTx->uint16_vRx_Counter++;
                                    // 受信データ数インクリメント
    }


    // 次の受信割り込みによるデータ取得の準備
//    HAL_UART_Receive_IT(&Handle_UART1,
    HAL_UART_Receive_IT(huart,
                pstructParam_Handle_UARTx->uint8_aBuffer_UART, 1);
}



//----------------------------------------------------------
// UARTx通信試験 : Error割り込み処理
//----------------------------------------------------------
// 引数:
// UART_HandleTypeDef *huart : UART_Handle構造体のポインタ

// struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
//              Handle UART用パラメータの構造体のポインタ


// 戻り値:
//    なし
//----------------------------------------------------------
void Test_UARTx_ErrorCallback(UART_HandleTypeDef *huart,
        struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx)
{
    if(pstructParam_Handle_UARTx->int16_vUART_InitDone == 1){
      HAL_UART_DeInit(huart);

      pstructParam_Handle_UARTx->int16_vUART_InitDone = 0;
                                        // 初期化なしをセット
      pstructParam_Handle_UARTx->uint16_vRx_Counter = 0;
                                        // 受信データ数カウンタクリア
    }
}



//----------------------------------------------------------
// UARTx通信試験 : 受信Timer割り込み処理
//----------------------------------------------------------
// 引数:
// struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
//              Handle UART用パラメータの構造体のポインタ


// 戻り値:
//    なし
//----------------------------------------------------------
void Test_UARTx_Interrupt_Timer(
        struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx)
{
  //--------------------------------------------------------
  // UART 受信Timer
  //--------------------------------------------------------
  // UART 受信TimeOut
  if(pstructParam_Handle_UARTx->int16_vUART_ReceiveTimeOut > 0){
    pstructParam_Handle_UARTx->int16_vUART_ReceiveTimeOut--;
  }

  // UART 受信終了Timer
  if(pstructParam_Handle_UARTx->int16_vUART_ReceiveEndTime > 0){
    pstructParam_Handle_UARTx->int16_vUART_ReceiveEndTime--;

    // 受信終了チェック
    if(pstructParam_Handle_UARTx->int16_vUART_ReceiveEndTime == 0){
      // 受信終了時に、受信データ書き込み位置をセーブする。
      pstructParam_Handle_UARTx->uint16_vRx_WritePointerSave
        = pstructParam_Handle_UARTx->uint16_vRx_WritePointer;
    }
  }
  //--------------------------------------------------------
}
