/*
 * Communication_Test_UARTx.h
 *
 *  Created on: 2024/11/17
 *      Author: egaono-electronics
 */

#ifndef COMMUNICATION_TEST_UARTX_H_
#define COMMUNICATION_TEST_UARTX_H_

#include "Dev_Conf.h"

#include def_Header_HAL

#include "H_Common_STM32F4_UART.h"



//-----------------------------------------------------------
// UARTx通信パラメータ初期化
//----------------------------------------------------------
// 引数:
// USART_TypeDef* USARTx :
//    where x can be 1, 2, 3, 4, 5 or 6
//    to select the USART or UART peripheral.

// UART_HandleTypeDef *pHandle_UARTx :
//  pointer to a UART_HandleTypeDef structure that contains
//  the configuration information
//                      for the specified UART module.

// uint32_t uint32_BaudRate : 通信速度

// struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
//              Handle UART用パラメータの構造体のポインタ


// 戻り値:
//    なし
//----------------------------------------------------------
void Initialize_Communication_UARTx(USART_TypeDef* USARTx,
                        UART_HandleTypeDef *pHandle_UARTx,
                        uint32_t uint32_BaudRate,
        struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx);



//------------------------------------------------------
// UARTx通信試験
//------------------------------------------------------
// 処理内容:

//  1. UARTの初期化がされていない場合、初期化します。
//  2. 受信が発生するのを待ち、受信データを取得します。
//  3. 受信データを判定して、内容に従い処理を行います。
//  4. 受信データによる処理の結果、
//          送信データがセットされた場合、送信処理を行います。
//----------------------------------------------------------
// 引数:
// uint8_t *puint8_JobNum : 処理番号が格納される変数のポインタ

// USART_TypeDef* USARTx :
//    where x can be 1, 2, 3, 4, 5 or 6
//    to select the USART or UART peripheral.

// UART_HandleTypeDef *pHandle_UARTx :
//  pointer to a UART_HandleTypeDef structure that contains
//  the configuration information
//                      for the specified UART module.

// uint8_t *puint8_aTx_Buffer : 送信Bufferのポインタ
// uint8_t *puint8_aRx_Buffer : 受信Bufferのポインタ

//  uint8_t *puint8_aReceiveData_UART :
//              UART通信処理用 受信データBufferのポインタ

// volatile uint16_t uint16_vTx_Transfer : 送信Bufferサイズ
// volatile uint16_t uint16_vRx_Receive : 受信Bufferサイズ

// struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
//              Handle UART用パラメータの構造体のポインタ


// 戻り値:
//   -1 : 処理中
//    0 : OK終了
//    1 : NG終了
//----------------------------------------------------------
int16_t Communication_Test_UARTx(uint8_t *puint8_JobNum,
                        USART_TypeDef* USARTx,
                        UART_HandleTypeDef *pHandle_UARTx,
                        uint8_t *puint8_aTx_Buffer,
                        uint8_t *puint8_aRx_Buffer,
                        uint8_t *puint8_aReceiveData_UART,
                        volatile uint16_t uint16_vTx_Transfer,
                        volatile uint16_t uint16_vRx_Receive,
        struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx);



//----------------------------------------------------------
// UARTx通信試験 : 送信割り込み処理
//----------------------------------------------------------
// 引数:
// struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
//              Handle UART用パラメータの構造体のポインタ


// 戻り値:
//    なし
//----------------------------------------------------------
void Test_UARTx_TxCpltCallback(
        struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx);



//----------------------------------------------------------
// UARTx通信試験 : 受信割り込み処理
//----------------------------------------------------------
// 引数:
// UART_HandleTypeDef *huart : UART_Handle構造体のポインタ

// struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
//              Handle UART用パラメータの構造体のポインタ


// 戻り値:
//    なし
//----------------------------------------------------------
void Test_UARTx_RxCpltCallback(UART_HandleTypeDef *huart,
        struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx);



//----------------------------------------------------------
// UARTx通信試験 : Error割り込み処理
//----------------------------------------------------------
// 引数:
// UART_HandleTypeDef *huart : UART_Handle構造体のポインタ

// struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
//              Handle UART用パラメータの構造体のポインタ


// 戻り値:
//    なし
//----------------------------------------------------------
void Test_UARTx_ErrorCallback(UART_HandleTypeDef *huart,
        struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx);



//----------------------------------------------------------
// UARTx通信試験 : 受信Timer割り込み処理
//----------------------------------------------------------
// 引数:
// struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx :
//              Handle UART用パラメータの構造体のポインタ


// 戻り値:
//    なし
//----------------------------------------------------------
void Test_UARTx_Interrupt_Timer(
        struct sParam_Handle_UARTx  *pstructParam_Handle_UARTx);


#endif /* COMMUNICATION_TEST_UARTX_H_ */
